function [press, u] = riemann(densityL,uL,pressL,cL,...
                                densityR,uR,pressR,cR);
%
% Compute pressure and velocity in the *star* region
%
global gamma g1 g2 g3 g4 g5 g6 g7 g8 g9;
tol = 1e-6;
deltaU = uR - uL;
% Compute critical velocity
deltaUCrit = g4*(cL+cR) - deltaU;
if( deltaUCrit <= 0 ) 
  error('Vacuum is generated by given data');
end
% Compute the initial guess
press = startE(densityL,uL,pressL,cL,...
                                densityR,uR,pressR,cR);
% Perform Newton root finding
press0 = press;
nIter = 20;
for i=1:nIter
  [fL, fLderiv] = prefun(press, densityL, pressL, cL);
  [fR, fRderiv] = prefun(press, densityR, pressR, cR);  
  press = press - (fL+fR+deltaU)/(fLderiv+fRderiv);
  if( 2*abs((press-press0)/(press+press0)) < tol )
    break;
  end
  if( i == nIter ) 
    error('Newton root finding failed');
  end
  if( press < 0 ) press = tol; end
  press0 = press;
end
u = (uL + uR + (fR-fL))/2;
return;
