/*
  Id:		$Id: GridConstructionX.C.sed,v 1.1.2.2 2002/03/20 00:43:45 gunney Exp $
  Copyright:	(c) 1997-2000 The Regents of the University of California
  Release:	$Name: v_0_1_7 $
  Revision:	$Revision: 1.1.2.2 $
  Modified:	$Date: 2002/03/20 00:43:45 $
  Description:	GridConstructionX class implementation
*/


#include <setArrayData.h>
#include <MultiDimArrayAccess.h>
#include <arrayConversionX.h>
#include <printObjectX.h>
#include "GridConstructionX.h"

#include <hier_Variable2.h>
#define hier_VariableX SAMRAI::hier_Variable2
#include <hier_VariableDatabase2.h>
#define hier_VariableDatabaseX SAMRAI::hier_VariableDatabase2
#include <pdat_CellData2.h>
#define pdat_CellDataX SAMRAI::pdat_CellData2
#include <hier_Patch2.h>
#define hier_PatchX SAMRAI::hier_Patch2
#include <geom_CartesianPatchGeometry2.h>
#define geom_CartesianPatchGeometryX SAMRAI::geom_CartesianPatchGeometry2




GridConstructionX::GridConstructionX(
  const string &object_name
, tbox_Pointer<tbox_Database> database
)
: d_context_all("ALL")
, d_constitutive("solution:constitutive",2)
, d_scalar("solution:scalar",1)
, d_vector("solution:vector",3)
{

  /*
    Register variables with VariableDatabase and get the descriptor indices
    for those variables.
  */
  {
    tbox_Pointer<hier_VariableContext> context_all_ptr(&d_context_all,false);
    hier_VariableDatabaseX *variable_db = hier_VariableDatabaseX::getDatabase();
    {
      int constitutive_id, scalar_id, vector_id;
      /*
	registerVariableAndContext returns the index unique to the
	vector-context pair.  It is not necessary to save it, because
	it can always be retrieved by  a call to
	variable_db->mapVariableAndContextToIndex.
	We just save it here to emphasize that it can be done.
      */
      constitutive_id
	= variable_db->registerVariableAndContext(
	  tbox_Pointer<hier_VariableX>(&d_constitutive,false)
	  , context_all_ptr );
      scalar_id
	= variable_db->registerVariableAndContext(
	  tbox_Pointer<hier_VariableX>(&d_scalar,false)
	  , context_all_ptr );
      vector_id
	= variable_db->registerVariableAndContext(
	  tbox_Pointer<hier_VariableX>(&d_vector,false)
	  , context_all_ptr );
    }
  }

  return;
}







void GridConstructionX::initializeLevelData (
    /*! Hierarchy to initialize */
    const tbox_Pointer< hier_PatchHierarchyX > hierarchy
  , /*! Level to initialize */
    const int level_number
  , const double init_data_time
  , const bool can_be_refined
  , /*! Whether level is being introduced for the first time */
    const bool initial_time
  , /*! Level to copy data from */
    const tbox_Pointer< hier_PatchLevelX > old_level
  , const bool allocate_data
  ) {

  /*
    Reference the level object with the given index from the hierarchy.
  */
  tbox_Pointer<hier_PatchLevelX> level
    = hierarchy->getPatchLevel(level_number);

  /*
    Get the index for the data.
  */
  int constitutive_id, scalar_id, vector_id;
  {
    hier_VariableDatabaseX *variable_db = hier_VariableDatabaseX::getDatabase();
    tbox_Pointer<hier_VariableContext> context_all_ptr(&d_context_all,false);
    constitutive_id =
      variable_db->mapVariableAndContextToIndex(
	  tbox_Pointer<pdat_CellVariableX<double> >(&d_constitutive,false)
	, context_all_ptr );
    scalar_id =
      variable_db->mapVariableAndContextToIndex(
	  tbox_Pointer<pdat_CellVariableX<double> >(&d_scalar,false)
	, context_all_ptr );
    vector_id =
      variable_db->mapVariableAndContextToIndex(
	  tbox_Pointer<pdat_CellVariableX<double> >(&d_vector,false)
	, context_all_ptr );
  }

  /*
    If instructed, allocate all patch data on the level.
  */
  if ( allocate_data ) {
    level->allocatePatchData( constitutive_id );
    level->allocatePatchData( scalar_id );
    level->allocatePatchData( vector_id );
  }

  /*
    Initialize data in all patches in the level.
  */
  int npatch = level->getNumberOfPatches();
  for ( int j=0; j<npatch; j++ ) {
    tbox_Pointer<hier_PatchX> patch
      = level->getPatch(j);
    tbox_Pointer<geom_CartesianPatchGeometryX> patch_geom
      = patch->getPatchGeometry();
    tbox_Pointer<pdat_CellDataX<double> > constitutive_data
      = patch->getPatchData(constitutive_id);
    tbox_Pointer<pdat_CellDataX<double> > scalar_data
      = patch->getPatchData(scalar_id);
    tbox_Pointer<pdat_CellDataX<double> > vector_data
      = patch->getPatchData(vector_id);
    MultiDimArrayAccess<double,2> t0
      = arrayData2ArrayAccess( constitutive_data->getArrayData() , 0 );
    setArrayDataToConstant(
	t0
      , constitutive_data->getGhostBox().lower()
      , constitutive_data->getGhostBox().upper()
      , patch_geom->getXLower()
      , patch_geom->getXUpper()
      , patch_geom->getDx()
      , double(level_number)
      );
    MultiDimArrayAccess<double,2> t1
      = arrayData2ArrayAccess( constitutive_data->getArrayData() , 1 );
    setArrayDataToConstant(
	t1
      , constitutive_data->getGhostBox().lower()
      , constitutive_data->getGhostBox().upper()
      , patch_geom->getXLower()
      , patch_geom->getXUpper()
      , patch_geom->getDx()
      , double(level_number)
      );
    MultiDimArrayAccess<double,2> t2
      = arrayData2ArrayAccess( scalar_data->getArrayData() , 0 );
    setArrayDataToSinusoidal(
        t2 // arrayData2ArrayAccess( scalar_data->getArrayData() , 0 )
      , scalar_data->getGhostBox().lower()
      , scalar_data->getGhostBox().upper()
      , patch_geom->getXLower()
      , patch_geom->getXUpper()
      , patch_geom->getDx()
      );
    setArrayDataToSinusoidalGradient(
        vector_data->getPointer(0)
      , vector_data->getPointer(1)
      , scalar_data->getGhostBox().lower()
      , scalar_data->getGhostBox().upper()
      , patch_geom->getXLower()
      , patch_geom->getXUpper()
      , patch_geom->getDx()
      );
    constitutive_data->getArrayData().getPointer(0);
    assert( constitutive_data->getArrayData().getPointer(0) );
    vector_data->getArrayData().getPointer(0);
    assert( vector_data->getArrayData().getPointer(0) );
    scalar_data->getArrayData().getPointer(0);
    assert( scalar_data->getArrayData().getPointer(0) );
  }
}




int GridConstructionX::registerVariablesWithPlotter(
  tbox_Pointer<plot_CartesianVizamraiDataWriterX> viz_writer
) {

  /*
    Get the indices for the data.
  */
  int constitutive_id, scalar_id, vector_id;
  {
    hier_VariableDatabaseX *variable_db = hier_VariableDatabaseX::getDatabase();
    tbox_Pointer<hier_VariableContext> context_all_ptr(&d_context_all,false);
    constitutive_id =
      variable_db->mapVariableAndContextToIndex(
	  tbox_Pointer<pdat_CellVariableX<double> >(&d_constitutive,false)
	, context_all_ptr );
    scalar_id =
      variable_db->mapVariableAndContextToIndex(
	  tbox_Pointer<pdat_CellVariableX<double> >(&d_scalar,false)
	, context_all_ptr );
    vector_id =
      variable_db->mapVariableAndContextToIndex(
	  tbox_Pointer<pdat_CellVariableX<double> >(&d_vector,false)
	, context_all_ptr );
  }

  /*
    Register variables with plotter.
  */
  viz_writer->registerPlotVariable("Constitutive variables", constitutive_id);
  viz_writer->registerPlotVariable("A scalar variable", scalar_id);
  viz_writer->registerPlotVariable("A vector variable", vector_id);

  return 0;
}
