/*
  Id:		$Id: GridConstructionX.h.sed,v 1.1.2.2 2002/04/12 22:28:03 gunney Exp $
  Copyright:	(c) 1997-2000 The Regents of the University of California
  Release:	$Name: v_0_1_7 $
  Revision:	$Revision: 1.1.2.2 $
  Modified:	$Date: 2002/04/12 22:28:03 $
  Description:	GridConstructionX class declaration
*/

#ifndef included_GridConstructionX_h
#define included_GridConstructionX_h


#include <string.h>
#include <tbox_Pointer.h>
#include <tbox_Database.h>


/*
  SAMRAI classes
*/
#include <mesh_StandardTagAndInitStrategy2.h>
#include <plot_CartesianVizamraiDataWriter2.h>
#include <pdat_CellVariable2.h>
#include <hier_Patch2.h>
#include <hier_PatchHierarchy2.h>
#include <hier_PatchLevel2.h>
#include <hier_IntVector2.h>
#include <mesh_StandardTagAndInitStrategy2.h>


using namespace SAMRAI;


/*!
  \brief Class to implement variables on a SAMR grid.

  This class demonstrates how to construct a grid, create variables
  on it and write the variables out for plotting.

  This class inherits and implements virtual functions from
  SAMRAI::mesh_StandardTagAndInitStrategyX to initialize data
  on the SAMR grid.
*/
class GridConstructionX
  : public mesh_StandardTagAndInitStrategyX
{

public:

  /*!
    \brief Constructor.

    Requirements:
    - the referenced objects

    Actions:
    - Initialize the viz_writer to serve this object
  */
  GridConstructionX(
    /*! Ojbect name */
    const string &object_name
  , /*! Input database */
    tbox_Pointer<SAMRAI::tbox_Database> database
  );



  //@{ \name SAMRAI::mesh_StandardTagAndInitStrategyX virtuals

public:

  /*!
    \brief Allocate and initialize data for a new level in the patch hierarchy.

    This is where you implement the code for initialize data on the
    grid.  Nevermind when it is called or where in the program that
    happens.  All the information you need to initialize the grid
    are in the arguments.

    \see SAMRAI::mesh_StandardTagAndInitStrategyX::initializeLevelData()
  */
  virtual void initializeLevelData (
      /*! Hierarchy to initialize */
      const tbox_Pointer< SAMRAI::hier_PatchHierarchyX > hierarchy
    , /*! Level to initialize */
      const int level_number
    , const double init_data_time
    , const bool can_be_refined
    , /*! Whether level is being introduced for the first time */
      const bool initial_time
    , /*! Level to copy data from */
      const tbox_Pointer< SAMRAI::hier_PatchLevelX > old_level
        = tbox_Pointer< SAMRAI::hier_PatchLevelX >((0))
    , const bool allocate_data = true
    );

  virtual void resetHierarchyConfiguration (
      const SAMRAI::tbox_Pointer< SAMRAI::hier_PatchHierarchyX > hierarchy
    , const int coarsest_level
    , const int finest_level
    ) { /* Nothing is needed. */ }

//@}




public:
  /*!
    \brief Tell a plotter which data to write for this class.
  */
  int registerVariablesWithPlotter(
    /*! Vizramrai writer */
    SAMRAI::tbox_Pointer<SAMRAI::plot_CartesianVizamraiDataWriterX> viz_writer
    );




  //@{
private:
  /*!
    \name Private state variables for solution.

    Included in this group are examples of the types of
    variables that can live on a SAMRAI mesh.

    Nevermind the significance of d_context_all for now.
    Contexts are not usefully employed in this tutorial.
  */

  /*!
    \brief Context for everything

    Special contexts are not used here, so get just one context for
    everythying.
  */
  SAMRAI::hier_VariableContext d_context_all;

  /*!
    \brief Constitutive variable.

    This could be some grid-fixed variable (such as permitivity and
    permeability in the case of electromagnetics).  We will actually
    set it so something that is usefull in this tutorial: the level
    and patch numbers.
  */
  SAMRAI::pdat_CellVariableX<double> d_constitutive;

  /*!
    \brief Scalar variable.
  */
  SAMRAI::pdat_CellVariableX<double> d_scalar;

  /*!
    \brief Vector variable.
  */
  SAMRAI::pdat_CellVariableX<double> d_vector;

  //@}

};


#endif	// included_GridConstructionX_h
