#include "setArrayData.h"
#include "MultiDimArrayAccess.h"
#include <math.h>
#include <iostream.h>



void setArrayDataToConstant(
  MultiDimArrayAccess<double,2> &s
  , const int *lower, const int *upper
  , const double *xlo, const double *xhi, const double *dx
  , double value
) {
  for ( int jj=lower[1]; jj<=upper[1]; ++jj) {
    for ( int ii=lower[0]; ii<=upper[0]; ++ii ) {
      s(jj,ii) = value;
    }
  }
}




void setArrayDataToCoordinateSum(
  double *ptr
  , const int *lower
  , const int *upper
  , const double *xlo, const double *xhi, const double *dx
) {
  MultiDimArrayAccess<double,2> s( ptr , lower , upper , true );
  for ( int jj=lower[1]; jj<=upper[1]; ++jj) {
    double y = xlo[1] + dx[1]*(jj-lower[1]+0.5);
    for ( int ii=lower[0]; ii<=upper[0]; ++ii ) {
      double x = xlo[0] + dx[0]*(ii-lower[0]+0.5);
      s(jj,ii) = x + y;
    }
  }
}




void setArrayDataToSinusoidal(
  MultiDimArrayAccess<double,2> &s
  , const int *lower
  , const int *upper
  , const double *xlo, const double *xhi, const double *dx
) {
  for ( int jj=lower[1]; jj<=upper[1]; ++jj) {
    double y = xlo[1] + dx[1]*(jj-lower[1]+0.5);
    double siny = sin(2*M_PI*y);
    for ( int ii=lower[0]; ii<=upper[0]; ++ii ) {
      double x = xlo[0] + dx[0]*(ii-lower[0]+0.5);
      double sinx = sin(2*M_PI*x);
      s(jj,ii) = sinx * siny;
    }
  }
}




void setArrayDataToSinusoidalGradient(
    double *gx_ptr, double *gy_ptr
  , const int *lower
  , const int *upper
  , const double *xlo, const double *xhi, const double *dx
) {
  MultiDimArrayAccess<double,2> gx( gx_ptr , lower , upper , true );
  MultiDimArrayAccess<double,2> gy( gy_ptr , lower , upper , true );
  for ( int jj=lower[1]; jj<=upper[1]; ++jj) {
    double y = xlo[1] + dx[1]*(jj-lower[1]+0.5);
    double siny = sin(2*M_PI*y);
    double cosy = cos(2*M_PI*y);
    for ( int ii=lower[0]; ii<=upper[0]; ++ii ) {
      double x = xlo[0] + dx[0]*(ii-lower[0]+0.5);
      double sinx = sin(2*M_PI*x);
      double cosx = cos(2*M_PI*x);
      gx(jj,ii) = 2*M_PI*cosx * siny;
      gy(jj,ii) = sinx * 2*M_PI*cosy;
    }
  }
}




void setArrayDataToScaled(
  MultiDimArrayAccess<double,2> &s
  , const int *lower, const int *upper
  , double factor
) {
  for ( int jj=lower[1]; jj<=upper[1]; ++jj) {
    for ( int ii=lower[0]; ii<=upper[0]; ++ii ) {
      s(jj,ii) *= factor;
    }
  }
}
