

#include "computeNormsX.h"
#include "arrayConversionX.h"

#include <tbox_Pointer.h>

#include <hier_PatchX.h>
#include <hier_PatchLevelX.h>
#include <hier_PatchHierarchyX.h>
#include <pdat_CellDataX.h>

#include <MultiDimArrayAccess.h>

using namespace SAMRAI;

int computeNormsX(
  const tbox_Pointer< SAMRAI::hier_PatchHierarchyX > hierarchy
, int id1
, int id2
, int idw
, double *l2norm
, double *linorm
) {
  /*
    Compute error on all levels, all patches.
  */
  double diff=0, difflinf=0, diffl2=0, weightsum=0;
  for ( int ln = hierarchy->getFinestLevelNumber(); ln >= 0; ln -- ) {

    tbox_Pointer<hier_PatchLevelX> level = hierarchy->getPatchLevel(ln);

    for ( hier_PatchLevelX::Iterator p(level); p; p++ ) {

      tbox_Pointer<hier_PatchX> patch = level->getPatch(p());

      /*
	Get information about the patch geometry.
      */
      // tbox_Pointer<geom_CartesianPatchGeometryX> patch_geom
	// = patch->getPatchGeometry();

      /*
	Get the patch data.
      */
      tbox_Pointer<pdat_CellDataX<double> > data1 = patch->getPatchData(id1);
      tbox_Pointer<pdat_CellDataX<double> > data2 = patch->getPatchData(id2);
      tbox_Pointer<pdat_CellDataX<double> > weight = patch->getPatchData(idw);

      {
	MultiDimArrayAccess<double,NDIM> ex =
          arrayData2ArrayAccess( data2->getArrayData() );
	ConstMultiDimArrayAccess<double,NDIM> co =
          arrayData2ArrayAccess( data1->getArrayData() );
	ConstMultiDimArrayAccess<double,NDIM> wt =
          arrayData2ArrayAccess( weight->getArrayData() );
        const int *lower = data1->getBox().lower();
        const int *upper = data1->getBox().upper();
#if NDIM == 1
	for ( int i=lower[0]; i<=upper[0]; ++i ) {
	  diff = fabs( co(i) - ex(i) );
	  if ( difflinf < diff ) difflinf = diff;
	  diffl2 += wt(i)*diff*diff;
	  weightsum += wt(i);
	}
#elif NDIM == 2
	for ( int j=lower[1]; j<=upper[1]; ++j ) {
	  for ( int i=lower[0]; i<=upper[0]; ++i ) {
	    diff = fabs( co(j,i) - ex(j,i) );
	    if ( difflinf < diff ) difflinf = diff;
	    diffl2 += wt(j,i)*diff*diff;
	    weightsum += wt(j,i);
	  }
	}
#elif NDIM == 3
	for ( int k=lower[1]; k<=upper[1]; ++k ) {
	  for ( int j=lower[1]; j<=upper[1]; ++j ) {
	    for ( int i=lower[0]; i<=upper[0]; ++i ) {
	      diff = fabs( co(k,j,i) - ex(k,j,i) );
	      if ( difflinf < diff ) difflinf = diff;
	      diffl2 += wt(k,j,i)*diff*diff;
	      weightsum += wt(k,j,i);
	    }
	  }
	}
#endif
      }

    }

  }
  diffl2 = sqrt( diffl2/weightsum );

  return 0;
}
