


#ifndef included_ObjectPrinting_h
#define included_ObjectPrinting_h

#include "printObjectX.h"

#include <MultiDimArrayAccess.h>

#include <SAMRAI_config.h>
#include <tbox_Pointer.h>
#include <hier_PatchHierarchyX.h>
#include <hier_PatchLevelX.h>
#include <hier_PatchDataX.h>
#include <hier_BoxX.h>
#include <pdat_ArrayDataX.h>


using namespace SAMRAI;


/*!
  \brief Print a patch hierarchy
*/
int printObjectX(
  ostream &os
, const hier_PatchHierarchyX &hierarchy
) {
  int nlevels = hierarchy.getNumberLevels();
  os << "\nhier_PatchHierarchyX::printClassData... " << endl;
  os << "Number of levels = " << nlevels << "\n";
  return 0;
}


/*!
  \brief Print a patch hierarchy
*/
int printObjectX(
  ostream &os
, const hier_PatchLevelX &level
) {
  int npatch = level.getNumberOfPatches();
  os << "\nhier_PatchLevelX::printClassData... " << endl;
  os << "Number of patches = " << npatch << "\n";
  return 0;
}


/*!
  \brief Print a box
*/
int printObjectX(
  ostream &os
, const hier_BoxX &box
) {
  os << "\nhier_BoxX::printClassData... " << endl;
  os << "( " << box.lower(0) << ' ' << box.lower(1)
     << " ) to ( "
     << box.upper(0) << ' ' << box.upper(1)
     << " )\n";
  return 0;
}


/*!
  \brief Print a patch
*/
int printObjectX(
  ostream &os
, const hier_PatchDataX &pdat
) {
  os << "\nhier_PatchX::printClassData... " << endl;
  const hier_BoxX &rbox = pdat.getBox();
  const hier_BoxX &gbox = pdat.getGhostBox();
  os << "Box = ( "
     << rbox.lower(0) << ' ' << rbox.lower(1)
     << " ) to ( "
     << rbox.upper(0) << ' ' << rbox.upper(1)
     << " )\n";
  os << "Ghost Box = ( "
     << gbox.lower(0) << ' ' << gbox.lower(1)
     << " ) to ( "
     << gbox.upper(0) << ' ' << gbox.upper(1)
     << " )\n";
  return 0;
}


/*!
  \brief Print an array data object
*/
template <class T>
int printObjectX(
  ostream &os
, const pdat_ArrayDataX<T> &adat
) {
  os << "\nhier_ArrayX::printClassData... " << endl;
  const hier_BoxX &rbox = adat.getBox();
  os << "Box = ( "
     << rbox.lower(0) << ' ' << rbox.lower(1)
     << " ) to ( "
     << rbox.upper(0) << ' ' << rbox.upper(1)
     << " )\n";
  printObjectX( os
	     , adat.getPointer()
	     , (const int*)adat.getBox().lower()
	     , (const int*)adat.getBox().upper()
	     );
  return 0;
}
template int printObjectX( ostream &os , const pdat_ArrayDataX<double> &adat );


/*!
  \brief Print an array
*/
int printObjectX(
  ostream &os
, const double *a_ptr , const int *a_lower , const int *a_upper
, const int *lower=NULL
, const int *upper=NULL
) {
  ConstMultiDimArrayAccess<double,NDIM> a( a_ptr , a_lower , a_upper , true );
  if ( ! lower ) lower = a_lower;
  if ( ! upper ) upper = a_upper;
  os << "\narray data... " << endl;
  for ( int j=lower[1]; j<=upper[1]; ++j ) {
    for ( int i=lower[0]; i<=upper[0]; ++i ) {
      os << i << ' ' << j << ' ' << a(j,i) << endl;
    }
  }
  return 0;
}

#endif	// included_ObjectPrinting_h
