


#include <tbox_Pointer.h>

#include <hier_PatchHierarchyX.h>
#include <hier_PatchLevelX.h>
#include <hier_PatchX.h>
#include <pdat_CellDataX.h>
#include <geom_CartesianPatchGeometryX.h>
#include <hier_BoxArrayX.h>
#include <hier_IntVectorX.h>
#include <hier_BoxX.h>

#include "setVectorWeightsX.h"


using namespace SAMRAI;


void setVectorWeightsX(
  SAMRAI::tbox_Pointer<hier_PatchHierarchyX> hierarchy
, int weight_id
) {
   for ( int amr_level = hierarchy->getFinestLevelNumber();
             amr_level >= 0;
             amr_level-- ) {

     /*
      * On every level, first assign cell volume to vector weight.
      */

      SAMRAI::tbox_Pointer<hier_PatchLevelX> level = hierarchy->getPatchLevel(amr_level);
      for (hier_PatchLevelX::Iterator p(level); p; p++) {
         SAMRAI::tbox_Pointer<hier_PatchX> patch = level->getPatch(p());
         SAMRAI::tbox_Pointer<geom_CartesianPatchGeometryX> patch_geometry =
            patch->getPatchGeometry();
         const double* dx = patch_geometry->getDx();
         double cell_vol = dx[0]*dx[1];
         SAMRAI::tbox_Pointer< pdat_CellDataX<double> > w =
            patch->getPatchData(weight_id);
         w->fillAll(cell_vol); 
      }

      /*
       * On all but the finest level, assign 0 to vector
       * weight to cells covered by finer cells.
       */

      if (amr_level < hierarchy->getFinestLevelNumber()) {

	/*
	 * First get the boxes that describe index space of the next finer 
         * level and coarsen them to describe corresponding index space 
         * at this level.
	 */

	 SAMRAI::tbox_Pointer<hier_PatchLevelX> next_finer_level = 
            hierarchy->getPatchLevel(amr_level+1);
         hier_BoxArrayX coarsened_boxes = next_finer_level->getBoxes(); 
         hier_IntVectorX coarsen_ratio = next_finer_level->getRatio();
         coarsen_ratio /= level->getRatio();
         coarsened_boxes.coarsen(coarsen_ratio);

	 /*
	  * Then set vector weight to 0 wherever there is
	  * a nonempty intersection with the next finer level.
	  * Note that all assignments are local.
	  */

         for (hier_PatchLevelX::Iterator p(level); p; p++) {

            SAMRAI::tbox_Pointer<hier_PatchX> patch = level->getPatch(p());
            for ( int i = 0; i < coarsened_boxes.getNumberOfBoxes(); i++ ) {

               hier_BoxX coarse_box = coarsened_boxes(i);
               hier_BoxX intersection = coarse_box*(patch->getBox());
               if ( !intersection.empty() ) {
                  SAMRAI::tbox_Pointer< pdat_CellDataX<double> > w =
                     patch->getPatchData(weight_id);
                  w->fillAll(0.0, intersection);

               }  // assignment only in non-empty intersection
            }  // loop over coarsened boxes from finer level
         }  // loop over patches in level
      }  // all levels except finest
   }  // loop over levels
}
